# 要件定義書

## プロジェクトの目的・背景

Flexifyは、YAML/JSONファイルで定義されたワークフローを柔軟に実行できるPythonライブラリです。複雑なタスクを再利用可能なモジュールとして定義し、設定ファイルで処理フローを記述することで、プログラムのハードコーディングを最小限に抑え、処理の流れを可視化・管理しやすくします。

### 背景
- 多くのプロジェクトで同様の処理パターンが繰り返し実装されている
- 処理の流れがソースコードに埋め込まれ、変更が困難
- ワークフローの可視化や進捗管理が難しい
- 既存のワークフローエンジンは大規模で複雑すぎる場合がある

## 利用者と困りごと

| 利用者の種類 | 利用者のゴール | 利用者の制約 | 利用者の困りごと |
|---|---|---|---|
| Python開発者 | 再利用可能な処理フローを簡単に構築したい | Pythonの基本知識はあるが、複雑なフレームワークは避けたい | ハードコーディングされた処理フローの保守が大変、処理の組み合わせを柔軟に変更できない |
| データサイエンティスト | データ処理パイプラインを構築・管理したい | YAML/JSONの基本知識、Python環境の構築が可能 | 前処理、モデル学習、評価などの手順を毎回コーディングするのが面倒 |
| システム管理者 | バッチ処理の流れを管理・監視したい | シンプルな設定ファイルで管理したい | 処理の進捗状況が把握しづらい、エラー時の対処が困難 |
| 将来のAPI利用者 | Web APIでワークフローを実行・管理したい | RESTful APIの基本知識 | ワークフローの実行状況をリアルタイムで確認できない |

## 採用する技術スタック

| 技術要素 | 採用技術 | 理由 |
|---|---|---|
| プログラミング言語 | Python 3.10+ | 豊富なライブラリ、データ処理に適している、最新の型ヒント機能 |
| パッケージ管理 | uv | 高速で信頼性の高いPythonパッケージ管理ツール |
| テストフレームワーク | pytest, pytest-cov | Pythonの標準的なテストツール |
| 設定ファイル形式 | YAML, JSON | 人間が読みやすく、標準的な形式 |
| 型ヒント | typing, dataclasses | コードの可読性と保守性の向上 |
| ドキュメント | Markdown | シンプルで管理しやすい |
| 将来：Web API | FastAPI | 高性能で型安全なAPI開発が可能 |
| 将来：分散処理 | Prefect/Airflow対応 | 既存のワークフローエンジンとの連携 |

## 機能（ユースケース）一覧

| 機能ID | 機能名 | 機能の説明 | 機能のイメージ |
|---|---|---|---|
| UC-01 | モジュール定義 | Moduleクラスを継承して再利用可能な処理単位を定義する | `class MyModule(Module): def execute(self, session): ...` |
| UC-02 | ワークフロー定義 | YAML/JSONファイルで処理の流れを定義する | `modules: [{name: "mod1", params: {...}}, ...]` |
| UC-03 | ワークフロー実行 | Runnerを使ってワークフローを実行する | `runner = SimpleRunner(); runner.run("workflow.yaml")` |
| UC-04 | 進捗状況確認 | 実行中のワークフローの進捗とステータスを確認する | `status = runner.get_status(); print(status.current_module)` |
| UC-05 | パラメータ検証 | モジュールの入出力パラメータを自動検証する | `param_info = MyModule.get_param_info()` |
| UC-06 | エラーハンドリング | エラー発生時の適切な処理とログ出力 | `try: runner.run() except FlexifyException as e: ...` |
| UC-07 | セッション管理 | モジュール間でのデータ共有と管理 | `session["key"] = value; next_module.execute(session)` |
| UC-08 | モジュール検索 | 利用可能なモジュールの自動検出と一覧表示 | `modules = ModuleRegistry.discover_modules()` |
| UC-09 | 将来：Web API実行 | HTTP経由でワークフローを実行・管理する | `POST /workflows/run {"workflow": "path/to/workflow.yaml"}` |
| UC-10 | 将来：分散実行 | Prefect/Airflowと連携して分散実行する | `runner = PrefectRunner(); runner.run_distributed()` |